package com.huaxia.bas.controller;


import com.huaxia.bas.service.UserService;
import com.huaxia.comm.domain.bas.UserInfo;
import com.ruoyi.common.core.domain.R;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * UserController
 *
 * @author sulingfeng
 * @since 2024/2/23 16:10
 */
@RestController
@RequestMapping("/bas/user")
@AllArgsConstructor
public class UserController {

    private UserService service;

    /**
     * 获取真实姓名
     *
     * @param userName 用户名
     * @return
     */
    @GetMapping("getRealName")
    public R<String> getRealName(@RequestParam String userName) {
        return R.ok(service.selectRealName(userName));
    }

    /**
     * 获取真实姓名
     *
     * @param userNames 用户名
     * @return
     */
    @PostMapping("getRealNames")
    public R<Map<String,String>> getRealNames(@RequestBody List<String> userNames) {
        return R.ok(service.selectRealNames(userNames));
    }

    /**
     * 获取用户信息
     *
     * @param userName 用户名
     * @return
     */
    @GetMapping("getUserInfo")
    public R<UserInfo> getUserInfo(@RequestParam String userName) {
        return R.ok(service.selectUserInfo(userName));
    }

    /**
     * 根据角色获取用户
     *
     * @param roleId 角色id
     * @return
     */
    @GetMapping("selectUserByRoleId")
    public R<List<String>> selectUserByRoleId(Long roleId) {
        return R.ok(service.selectUserByRoleId(roleId));
    }

    /**
     * 根据用户名或昵称获取用户
     * @param name
     * @return
     */
    @GetMapping("getInfoByName")
    public R<List<UserInfo>> getInfoByName(String name){
        return R.ok(service.getInfoByName(name));
    }


    /**
     * 根据部门获取用户
     * @param deptId 部门id
     * @return
     */
    @GetMapping("getInfoByDeptId")
    public R<List<UserInfo>> getInfoByDeptId(Long deptId,String name){
        return R.ok(service.getInfoByDeptId(deptId,name));
    }
}
