package com.huaxia.bas.service;

import com.huaxia.bas.mapper.UserMapper;
import com.huaxia.comm.domain.bas.UserInfo;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * UserService
 *
 * @author sulingfeng
 * @since 2024/2/23 16:14
 */
@AllArgsConstructor
@Service
@Transactional(rollbackFor = Exception.class)
public class UserService {

    private UserMapper mapper;

    /**
     * 获取真实姓名
     *
     * @param userName 用户名
     * @return
     */
    public String selectRealName(String userName) {
        return mapper.selectRealName(userName);
    }

    /**
     * 获取真实姓名
     *
     * @param userNames 用户名
     * @return
     */

    public Map<String,String> selectRealNames(List<String> userNames) {
        Map<String,String> res = new HashMap<>();
        for (String userName : userNames) {
            if (userName != null){
                String realName = mapper.selectRealName(userName);
                res.put(userName,realName);
            }
        }
        return res;
    }
    /**
     * 获取用户信息
     *
     * @param userName 用户名
     * @return
     */
    public UserInfo selectUserInfo(String userName) {
        return mapper.selectUserInfo(userName);
    }

    /**
     * 根据角色获取用户
     *
     * @param roleId 角色id
     * @return
     */
    public List<String > selectUserByRoleId(Long roleId){
        return mapper.selectUserByRoleId(roleId);
    }

    /**
     * 根据用户名或昵称获取用户
     * @param name
     * @return
     */
    public List<UserInfo> getInfoByName(String name) {
        return mapper.getInfoByName(name);
    }

    /**
     * 根据部门获取用户
     * @param deptId 部门id
     * @return
     */
    public List<UserInfo> getInfoByDeptId(Long deptId,String name) {
        return mapper.getInfoByDeptId(deptId,name);
    }
}
