package com.huaxia.comm.util;

import cn.hutool.core.lang.UUID;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.exception.base.BaseException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import org.apache.poi.ss.usermodel.PictureData;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;
import sun.misc.BASE64Encoder;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

import static org.springframework.util.DigestUtils.md5DigestAsHex;

/**
 * CommonExceUtil
 *
 * @author sulingfeng
 * @date 2023/8/29 10:52
 */
public class CommonExcelUtil {

    private static final Logger log = LoggerFactory.getLogger(CommonExcelUtil.class);

    /**
     * 编码文件名
     */
    public static String encodingFilename(String filename)
    {
        filename = filename + "-" + System.currentTimeMillis() + ".xlsx";
        return filename;
    }


    /**
     * 写入excel文件 并返回路径
     * @return
     */
    public static String exportExcel(String sheetName, Workbook workBook) {
        OutputStream out = null;
        try {
            String filename = encodingFilename(sheetName);
            String localFilename = getAbsoluteFile(filename);
            log.info("生成文件本地路径：{}", localFilename);
            out = new FileOutputStream(localFilename);
            workBook.write(out);
            return filename;
        } catch (Exception e) {
            log.error("导出Excel异常{}", e.getMessage());
            throw new BaseException("导出Excel失败，请联系网站管理员！");
        } finally {
            //清理临时文件
            if(workBook instanceof SXSSFWorkbook){
                ((SXSSFWorkbook) workBook).dispose();
            }
            if (workBook != null) {
                try {
                    workBook.close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }

        }
    }

    public static String convertExcelPicToBase64(PictureData pictureData) {

        String extension = "."+pictureData.suggestFileExtension();
        // 对字节数组进行Base64编码，得到Base64编码的字符串
        BASE64Encoder encoder = new BASE64Encoder();
        String base64Str = encoder.encode(pictureData.getData());
        StringBuffer prefix = getBase64Prefix(extension);
        return prefix.append(";base64,").append(base64Str).toString();
    }

    private static StringBuffer getBase64Prefix(String extension) {
        StringBuffer prefix = new StringBuffer("data:image/");
        if (".jpg".equalsIgnoreCase(extension) || ".jpeg".equalsIgnoreCase(extension)) {//data:image/jpeg;base64,base64编码的jpeg图片数据
            prefix.append("jpeg");
        } else if (".ico".equalsIgnoreCase(extension)) {//data:image/x-icon;base64,base64编码的jpeg图片数据
            prefix.append("x-icon");
        } else {
            prefix.append(extension.substring(1));
        }
        return prefix;
    }

    public static String getAbsoluteFile(String filename) {
        String downloadPath = RuoYiConfig.getDownloadPath() + filename;
        File desc = new File(downloadPath);
        if (!desc.getParentFile().exists()) {
            desc.getParentFile().mkdirs();
        }

        return downloadPath;
    }

    /**
     * base64转成文件
     * @param base64
     * @return filePath
     */
    public static String uploadPicture(String base64){
        String [] baseData = null;
        if(base64 == null || "".equals(base64)){
            throw new BaseException("上传失败，上传图片数据为空");
        }else{
            baseData = base64.split("base64,");
            if(baseData == null || baseData.length != 2){
                throw new BaseException("上传失败，数据不合法");
            }
        }
        //1.获取图片后缀
        String extension = CommonUtils.getBase64Extension(baseData[0]);
        //2.将图片转为文件
        String filePath = RuoYiConfig.getUploadPath() + "/"+ DateUtils.datePath() ;
        String fileName = IdUtils.fastUUID() + extension;
        CommonUtils.convertBase64ToFile(baseData[1], filePath, fileName);
        return CommonUtils.getUrlPath(filePath, fileName);
    }

    /**
     * base64转成文件 (excel导入)
     * @param
     * @return filePath
     */
    public static String uploadPictureByExcel(String base64,Integer index,StringBuilder message){
        String [] baseData = null;
        if(base64 == null || "".equals(base64)){
            message.append(String.format("第"+index+"行上传图片数据为空%n"));
        }else{
            baseData = base64.split("base64,");
            if(baseData == null || baseData.length != 2){
                message.append(String.format("第"+index+"行上传图片数据不合法%n"));
            }
        }
        //1.获取图片后缀
        String extension = CommonUtils.getBase64Extension(baseData[0]);
        //2.将图片转为文件
        String filePath = RuoYiConfig.getUploadPath() + "/"+ DateUtils.datePath() ;
        String fileName = IdUtils.fastUUID() + extension;
        CommonUtils.convertBase64ToFile(baseData[1], filePath, fileName);
        return CommonUtils.getUrlPath(filePath, fileName);
    }

    /**
     * 获取图片hash值
     * @param avatar base64
     * @return
     */
    public static String getHash(String avatar){
        Map<String, Object> retMap = parse(avatar, "");
        byte[] bytes = (byte[])retMap.get("bytes");
        // hash = md5(file bytes)
        return md5DigestAsHex(bytes);
    }

    /**
     * 解析base64字符串为文件字节数组
     * @param base64
     * @param renamed
     * @return
     */
    public static Map<String, Object> parse(String base64, String renamed) {
        String [] base64Ary;
        if(base64 == null || "".equals(base64.trim())){
            throw new BaseException("上传失败，上传图片数据为空");
        }else{
            base64Ary = base64.split("base64,");
            if(base64Ary == null || base64Ary.length != 2){
                throw new BaseException("上传失败，数据不合法");
            }
        }
        String base64Pre = base64Ary[0], base64Dat = base64Ary[1];
        //1.获取图片后缀
        String extension = CommonUtils.getBase64Extension(base64Pre);
        //2.将图片转为文件 因为BASE64Decoder的jar问题，此处使用spring框架提供的工具包
        byte[] bytes = javax.xml.bind.DatatypeConverter.parseBase64Binary(base64Dat); // Base64Utils.decodeFromString(base64Dat); //
        // 重命名后的文件名
        renamed = StringUtils.hasLength(renamed)? renamed: UUID.fastUUID().toString().replace("-", "");
        String fileName = renamed + extension;
        Map<String, Object> retMap = new HashMap<>();
        retMap.put("fileName", fileName);
        retMap.put("bytes", bytes);
        retMap.put("extension", extension);
        return retMap;
    }
}
