package com.huaxia.comm.util;

import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.exception.base.BaseException;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.PictureData;
import org.springframework.lang.Nullable;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.*;
import java.math.BigDecimal;
import java.net.URL;
import java.text.DecimalFormat;
import java.util.List;

/**
 * @author gls
 * @create 2021-11-12 18:26
 */
public class CommonUtils {

    public static final String UTF8 = "UTF-8";
    public static final String GBK = "GBK";
    public static final String HTTP = "http://";
    public static final String HTTPS = "https://";
    public static final String SUCCESS = "0";
    public static final String FAIL = "1";
    public static final String LOGIN_SUCCESS = "Success";
    public static final String LOGOUT = "Logout";
    public static final String LOGIN_FAIL = "Error";
    public static final String CAPTCHA_CODE_KEY = "captcha_codes:";
    public static final String LOGIN_TOKEN_KEY = "login_tokens:";
    public static final String REPEAT_SUBMIT_KEY = "repeat_submit:";
    public static final Integer CAPTCHA_EXPIRATION = 2;
    public static final String TOKEN = "token";
    public static final String TOKEN_PREFIX = "Bearer ";
    public static final String LOGIN_USER_KEY = "login_user_key";
    public static final String JWT_USERID = "userid";
    public static final String JWT_USERNAME = "sub";
    public static final String JWT_AVATAR = "avatar";
    public static final String JWT_CREATED = "created";
    public static final String JWT_AUTHORITIES = "authorities";
    public static final String SYS_CONFIG_KEY = "sys_config:";
    public static final String SYS_DICT_KEY = "sys_dict:";
    public static final String RESOURCE_PREFIX = "/profile";

    /**
     * 判断list是否为空
     * @param list
     * @return
     */
    public static boolean isListEmpty(List list) {
        if (list == null || list.size() <= 0) {
            return true;
        }
        return false;
    }

    /**
     * 判断值是否为空
     * @param str
     * @return
     */
    public static boolean isEmpty(@Nullable Object str) {
        return str == null || "".equals(str);
    }

    /**
     * 判断值是否为空
     * @param str
     * @return
     */
    public static boolean isNotEmpty(@Nullable Object str) {
        return str != null && !"".equals(str.toString().trim());
    }

    public static String getNotNullString(String s) {
        return s == null ? "" : s;
    }

    /**
     * 判断是否相同
     * @param oldVal
     * @param newVal
     * @return
     */
    public static Boolean isEqual(Object oldVal, Object newVal) {
        if (oldVal == null) {
            oldVal = "";
        }
        if (newVal == null) {
            newVal = "";
        }
        if (oldVal instanceof BigDecimal && newVal instanceof BigDecimal) {
            if (((BigDecimal) oldVal).compareTo((BigDecimal) newVal) == 0) {
                return true;
            }
            /*if("0.00".equals(oldVal.toString().trim())){
                oldVal = "0";
            }
            if("0.00".equals(newVal.toString().trim())){
                newVal = "0";
            }*/
        }
        if (oldVal.toString().trim().equals(newVal.toString().trim())) {
            return true;
        }
        return false;
    }

    /**
     * 本地文件（图片、excel等）转换成Base64字符串
     * @param imgPath
     */
    public static String convertUrlToBase64(String imgPath, String extension) {

        ByteArrayOutputStream outputStream;
        try {
            URL url = new URL(imgPath);
            BufferedImage bufferedImage = ImageIO.read(url);
            outputStream = new ByteArrayOutputStream();
            ImageIO.write(bufferedImage,extension.substring(1),outputStream);
        } catch (IOException e) {
            e.printStackTrace();
            return imgPath;
        }
        // 对字节数组进行Base64编码，得到Base64编码的字符串
        BASE64Encoder encoder = new BASE64Encoder();
        String base64Str = encoder.encode(outputStream.toByteArray());
        StringBuffer prefix = getBase64Prefix(extension);
        return prefix.append(";base64,").append(base64Str).toString();
    }

    private static StringBuffer getBase64Prefix(String extension) {
        StringBuffer prefix = new StringBuffer("data:image/");
        if (".jpg".equalsIgnoreCase(extension) || ".jpeg".equalsIgnoreCase(extension)) {//data:image/jpeg;base64,base64编码的jpeg图片数据
            prefix.append("jpeg");
        } else if (".ico".equalsIgnoreCase(extension)) {//data:image/x-icon;base64,base64编码的jpeg图片数据
            prefix.append("x-icon");
        } else {
            prefix.append(extension.substring(1));
        }
        return prefix;
    }

    public static String convertExcelPicToBase64(PictureData pictureData) {

        String extension = "."+pictureData.suggestFileExtension();
        // 对字节数组进行Base64编码，得到Base64编码的字符串
        BASE64Encoder encoder = new BASE64Encoder();
        String base64Str = encoder.encode(pictureData.getData());
        StringBuffer prefix = getBase64Prefix(extension);
        return prefix.append(";base64,").append(base64Str).toString();
    }

    /**
     * 将base64字符串，生成文件
     */
    public static File convertBase64ToFile(String base64Data, String filePath, String fileName) {

        BufferedOutputStream bos = null;
        FileOutputStream fos = null;
        File file = null;
        try {

            BASE64Decoder decoder = new BASE64Decoder();
            byte[] bfile = decoder.decodeBuffer(base64Data);

            file = new File(filePath + File.separator + fileName);
            //判断文件目录是否存在
            if (!file.getParentFile().exists()) {
                file.getParentFile().mkdirs();
            }
            fos = new FileOutputStream(file);
            bos = new BufferedOutputStream(fos);
            bos.write(bfile);
            return file;
        } catch (Exception e) {
            e.printStackTrace();
            throw new BaseException("base64转图片失败！");
        } finally {
            if (bos != null) {
                try {
                    bos.close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
            if (fos != null) {
                try {
                    fos.close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
        }
    }

    public static final File getAbsoluteFile(String uploadDir, String fileName) throws IOException {
        File desc = new File(uploadDir + File.separator + fileName);

        if (!desc.getParentFile().exists()) {
            desc.getParentFile().mkdirs();
        }
        if (!desc.exists()) {
            desc.createNewFile();
        }
        return desc;
    }

    /**
     * 获取文件扩展名
     * @return
     */
    public static String getExtension(String fileName) {
        String extension = "";
        int index = fileName.lastIndexOf(".");
        if (index != -1) {
            extension = fileName.substring(index);
        }
        return extension;
    }

    public static String getNotDotExtension(String fileName) {
        String extension = "";
        int index = fileName.lastIndexOf(".");
        if (index != -1) {
            extension = fileName.substring(index + 1);
        }
        return extension;
    }

    /**
     * 获取base64图片扩展名
     * @return
     */
    public static String getBase64Extension(String base64Data) {

        String extension = "";
        if ("data:image/jpeg;".equalsIgnoreCase(base64Data)) {//data:image/jpeg;base64,base64编码的jpeg图片数据
            extension = ".jpg";
        }else if ("data:image/jpg;".equalsIgnoreCase(base64Data)) {//data:image/jpg;base64,base64编码的icon图片数据
            extension = ".jpg";
        } else if ("data:image/x-icon;".equalsIgnoreCase(base64Data)) {//data:image/x-icon;base64,base64编码的icon图片数据
            extension = ".ico";
        } else if ("data:image/gif;".equalsIgnoreCase(base64Data)) {//data:image/gif;base64,base64编码的gif图片数据
            extension = ".gif";
        } else if ("data:image/png;".equalsIgnoreCase(base64Data)) {//data:image/png;base64,base64编码的png图片数据
            extension = ".png";
        } else {
            throw new BaseException("上传图片格式不合法");
        }
        return extension;
    }

    /**
     * 文件路径转为url
     * @param uploadDir
     * @param fileName
     * @return
     */
    public static String getUrlPath(String uploadDir, String fileName) {
        int dirLastIndex = RuoYiConfig.getProfile().length() + 1;
        String currentDir = StringUtils.substring(uploadDir, dirLastIndex);
        String pathFileName = RESOURCE_PREFIX + "/" + currentDir + "/" + fileName;
        return ServletUtils.getUrl(ServletUtils.getRequest())+ pathFileName;
    }
    

    /**
     * 数据库中存的url转为本地路径
     * @param url
     * @return
     */
    public static String getLocalPath(String url) {
        int dirLastIndex = (ServletUtils.getUrl() + RESOURCE_PREFIX).length() + 1;
        return RuoYiConfig.getProfile() + "/" + StringUtils.substring(url, dirLastIndex);
    }

    public static String getFileSize(File file) {
        if (file.exists() && file.isFile()) {
            return getFileSize(file.length());
        } else if (file.exists() && file.isDirectory()) {
            return "";
        } else {
            return "0B";
        }
    }

    public static String getFileSize(Long fileLength) {
        String size = "";
        DecimalFormat df = new DecimalFormat("#.00");
        if (fileLength < 1024) {
            size = df.format((double) fileLength) + "BT";
        } else if (fileLength < 1048576) {
            size = df.format((double) fileLength / 1024) + "KB";
        } else if (fileLength < 1073741824) {
            size = df.format((double) fileLength / 1048576) + "MB";
        } else {
            size = df.format((double) fileLength / 1073741824) + "GB";
        }
        return size;
    }

    /**
     * 清理小数点后面的尾数0
     * 注：此种需求通常是BigDecimal类型带有精度的情况下需要的
     * @param s
     * @author kangtengjiao 20220404
     * @return
     */
    public static String clearNumberZeroSuffix(String s) {
        if(s == null) return null;

        final int dotIdx = s.indexOf(".");

        if(dotIdx > 0) {
            String intStr = s.substring(0,dotIdx);

            String floatStr =s.substring(dotIdx+1);

            int suffixZeroCounter = 0;
            final int floatStrLen = floatStr.length();
            for(int  i = floatStrLen - 1; i >=0;  i--){

                if( '0' == floatStr.charAt(i) ) {
                    suffixZeroCounter ++;
                    continue;
                }
                break;
            }



            if(suffixZeroCounter > 0) {
                int realFloatLen = floatStrLen - suffixZeroCounter;
                floatStr = floatStr.substring(0, realFloatLen);
            }

            s = !"".equals(floatStr)? intStr + "." + floatStr : intStr;

        }

        return s;
    }
}
