package com.huaxia.comm.util;

import net.sourceforge.pinyin4j.PinyinHelper;
import net.sourceforge.pinyin4j.format.HanyuPinyinCaseType;
import net.sourceforge.pinyin4j.format.HanyuPinyinOutputFormat;
import net.sourceforge.pinyin4j.format.HanyuPinyinToneType;
import net.sourceforge.pinyin4j.format.HanyuPinyinVCharType;
import net.sourceforge.pinyin4j.format.exception.BadHanyuPinyinOutputFormatCombination;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * PinyinTool
 *
 * @author Administrator tengjiao
 * @date 2023/6/20 13:30
 */
public final class PinyinTool {
    /**
     * 取一个中文字符的首字母
     * @param chineseChar
     * @return
     */
    public static char getFirstSpell(char chineseChar) {
        String[] temp = PinyinHelper.toHanyuPinyinStringArray(chineseChar);
        if(temp != null && temp.length > 0) {
            return temp[0].charAt(0);
        }
        return '\0';
    }

    /**
     * 提取中文文字列表
     * @param line
     * @return
     */
    public static List<String> readChinese(String line) {
        List<String> result = new ArrayList<>();
        Pattern p = Pattern.compile("[\u4e00-\u9fa5]");
        Matcher m = p.matcher(line);
        while(m.find()){
            result.add(m.group());
        }
        return result;
    }

    /**
     * 获取汉字串拼音首字母
     * @param str 字符串
     * @param onlyChinese 是否仅提取中文，否则英文字符不变
     * @return 汉语拼音首字母
     */
    public static String getFirstSpell(String str, boolean onlyChinese) {
        StringBuilder strBuf = new StringBuilder();
        if(onlyChinese) {
            List<String> result = readChinese(str);
            for (String s : result) {
                strBuf.append(getFirstSpell(s.toCharArray()[0]));
            }
            return strBuf.toString().toUpperCase();
        } else {
            char[] arr = str.toCharArray();
            HanyuPinyinOutputFormat defaultFormat = new HanyuPinyinOutputFormat();
            defaultFormat.setCaseType(HanyuPinyinCaseType.UPPERCASE);
            defaultFormat.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
            for (int i = 0; i < arr.length; i++) {
                if (arr[i] > 128) {
                    try {
                        String[] temp = PinyinHelper.toHanyuPinyinStringArray(arr[i], defaultFormat);
                        if (temp != null && temp.length > 0) {
                            strBuf.append(temp[0].charAt(0));
                        }
                    } catch (BadHanyuPinyinOutputFormatCombination e) {
                        e.printStackTrace();
                    }
                } else {
                    strBuf.append(arr[i]);
                }
            }
            return strBuf.toString().replaceAll("\\W", "").trim();
        }
    }

    /**
     * 将字符串中的中文转化为拼音,英文字符不变
     *
     * @param inputString 汉字
     * @return
     */
    public static String getPingYin(String inputString) {
        HanyuPinyinOutputFormat format = new HanyuPinyinOutputFormat();
        format.setCaseType(HanyuPinyinCaseType.LOWERCASE);
        format.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
        format.setVCharType(HanyuPinyinVCharType.WITH_V);
        String output = "";
        if (null != inputString && 0 < inputString.length() && !"null".equals(inputString)) {
            char[] input = inputString.trim().toCharArray();
            try {
                for (int i = 0; i < input.length; i++) {
                    if (Character.toString(input[i]).matches("[\\u4E00-\\u9FA5]+")) {
                        String[] temp = PinyinHelper.toHanyuPinyinStringArray(input[i], format);
                        output += temp[0];
                    } else {
                        output += Character.toString(input[i]);
                    }
                }
            } catch (BadHanyuPinyinOutputFormatCombination e) {
                System.out.println("将字符串中的中文转化为拼音,英文字符不变异常 :" + e.getMessage());
            }
        } else {
            return "";
        }
        return output;
    }
}
