package com.huaxia.comm.util;

import org.springframework.util.Assert;
import org.springframework.util.ConcurrentReferenceHashMap;

import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * ReflectionTool
 *
 * @author kangtengjiao
 * @date 2023/12/4 12:01
 */
public final class ReflectionTool {
    private static final Field[] EMPTY_FIELD_ARRAY = new Field[0];
    private static final Map<Class<?>, Field[]> declaredFieldsCache = new ConcurrentReferenceHashMap(256);

    public static Field[] getDeclaredFields(Class<?> clazz) {
        Assert.notNull(clazz, "Class must not be null");
        Field[] result = declaredFieldsCache.get(clazz);
        if (result == null) {
            try {
                result = clazz.getDeclaredFields();
                declaredFieldsCache.put(clazz, result.length == 0 ? EMPTY_FIELD_ARRAY : result);
            } catch (Throwable var3) {
                throw new IllegalStateException("Failed to introspect Class [" + clazz.getName() + "] from ClassLoader [" + clazz.getClassLoader() + "]", var3);
            }
        }

        return result;
    }

    public static <T> Map<String, Field> getDeclaredFieldsMap(Class<T> clazz) {
        Map<String,Field> fieldMap = new HashMap<>();
        Arrays.stream(ReflectionTool.getDeclaredFields(clazz)).forEach(e->fieldMap.put(e.getName(), e));
        return fieldMap;
    }

    /**
     * 循环向上转型, 获取所有类型的字段
     * @param clazz : 子类
     * @return 父类中
     * */
    public static Field[] getAllDeclaredFields(Class clazz) {
        return declaredFieldsCache.computeIfAbsent(clazz, (key->{
            Class aClass = clazz;
            Map<String, Field> filedMap = new HashMap<>();
            for (; aClass != Object.class; aClass = aClass.getSuperclass()) {
                Field[] fields = aClass.getDeclaredFields();
                for(Field field: fields) {
                    if( filedMap.containsKey( field.getName() ) ) {
                        continue;
                    }
                    filedMap.put(field.getName(), field);
                }
            }
            return filedMap.values().toArray(new Field[filedMap.size()]);
        }));
    }

}
