package com.huaxia.imes.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huaxia.comm.domain.imes.Cutpiece;
import com.huaxia.comm.domain.imes.SysMeConfig;
import com.huaxia.imes.mapper.CutpieceMapper;
import com.huaxia.imes.mapper.SysMeConfigMapper;
import com.ruoyi.common.annotation.DataSource;
import com.ruoyi.common.enums.DataSourceType;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

/**
 * @author zx
 * @since 2024年10月18日
 */
@Service
@Slf4j
@AllArgsConstructor
@Transactional(rollbackFor = Exception.class)
@DataSource(DataSourceType.SLAVE)
public class CutpieceService extends ServiceImpl<CutpieceMapper, Cutpiece> implements IService<Cutpiece> {

    private CutpieceMapper cutpieceMapper;

    private SysMeConfigMapper sysMeConfigMapper;//配置信息


    /**
     * 获取生产
     *
     * @return
     */
    public List<Cutpiece> queryList() {
        //获取机器配置信息
        List<SysMeConfig> configs = sysMeConfigMapper.queryList(new SysMeConfig());
        Assert.isTrue(configs!=null&&configs.size()>0,"500-机器配置信息不存在,请先配置");
        //遍历配置信息
        List<Cutpiece> cutpieces =new ArrayList<>();
        for (SysMeConfig config : configs) {
            //获取机器编号 获取目标产量 获取上班时间
            String cutrecordNo = config.getTblMachineNo();//机器编号
            Double capacity = config.getCapacity();//目标产能
            LocalTime confiTime = config.getConfiTime();//上班时间
            // 获取当前日期
            LocalDate currDate = LocalDate.now();
            /*//减三天 todo(用来获取测试数据)
            currDate = currDate.minusDays(4);*/
            // 当天零点零分
            LocalDateTime startTime = currDate.atTime(LocalTime.MIN);
            //获取当天 23:59:59
            LocalDateTime endTime = currDate.atTime(LocalTime.MAX);
            //通过机器编号
            /*List<Cutpiece> bo = cutpieceMapper.queryList(dto);*/
            Cutpiece boy = cutpieceMapper.selectBoy(config.getTblMachineNo(), startTime, endTime);
            if (boy != null){
                boy.setTarget(capacity);//目标产能
                //获取切割用时  时间秒 换算成小时
                Long count = boy.getTotalTime();
                double totalTimeHoursWithFraction = count / 3600.0;
                DecimalFormat Format = new DecimalFormat("#.##");
                boy.setRunTime(Double.valueOf(Format.format(totalTimeHoursWithFraction)));
                //机器上班时间 通过当前年月日 拼接 时间
                String workTimeString = currDate.format(DateTimeFormatter.ISO_LOCAL_DATE) + " " + confiTime;
                LocalDateTime workTime = LocalDateTime.parse(workTimeString, DateTimeFormatter.ofPattern("yyyy-MM-dd H:mm"));
                boy.setWorkingTime(workTime);
                //机器 开始运作时间
                LocalDateTime workStartTime = boy.getStartTime();
                //机器 结束运作时间
                LocalDateTime workEndTime = boy.getEndTime();
                //计算机器运作时间致机器上班时间的差值
                Duration duration = Duration.between(workStartTime, workEndTime);
                //差值换算成秒
                long workTimeSeconds = duration.getSeconds();
                //机器大致运作时长-机器实际运作时长=机器闲置时长
                long idleTime = workTimeSeconds - count;
                //换算成小时
                double idleTimeHoursWithFraction = idleTime / 3600.0;
                //保留小数点后两位
                DecimalFormat format = new DecimalFormat("#.##");
                boy.setFreeTime(Double.valueOf(format.format(idleTimeHoursWithFraction)));
                /*log.info("机器闲置时间:{}", idleTimeHoursWithFraction);*/
                boy.setTarget(capacity);
                //换算百分比
                double completeRatePercentage = (boy.getCount() / capacity) * 100;
                BigDecimal bd = new BigDecimal(completeRatePercentage).setScale(2, RoundingMode.HALF_UP);
                double roundedCompleteRatePercentage = bd.doubleValue();
                boy.setFinish(roundedCompleteRatePercentage);
                /*log.info("完成率:{}%", completeRatePercentage);*/
                cutpieces.add(boy);
            }
        }
        return cutpieces;
    }
}
