package com.huaxia.imes.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huaxia.comm.domain.imes.Cutpiece;
import com.huaxia.comm.domain.imes.SysMeConfig;
import com.huaxia.imes.mapper.CutpieceMapper;
import com.huaxia.imes.mapper.SysMeConfigMapper;
import com.ruoyi.common.annotation.DataSource;
import com.ruoyi.common.enums.DataSourceType;
import com.ruoyi.common.utils.StringUtils;
import lombok.AllArgsConstructor;
import org.aspectj.weaver.ast.Var;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author zx
 * @since 2024年10月22日
 */
@Transactional(rollbackFor = Exception.class)
@Service
@AllArgsConstructor
@DataSource(DataSourceType.SLAVE)
public class SysMeConfigService extends ServiceImpl<SysMeConfigMapper, SysMeConfig> implements IService<SysMeConfig> {


    private SysMeConfigMapper sysMeConfigMapper;
    private CutpieceMapper cutpieceMapper;


    /**
     * 分页查询可配置信息
     *
     * @param boy
     * @param page
     * @param size
     * @return
     */
    public IPage<SysMeConfig> queryList(SysMeConfig boy, int page, int size) {
        int offset = (page - 1) * size;
        Map<String, Object> params = new HashMap<>();
        params.put("boy", boy);
        params.put("offset", offset);
        params.put("pageSize", size);

        // 获取总记录数
        int total = sysMeConfigMapper.countByParams(params);

        // 执行分页查询
        List<SysMeConfig> records = sysMeConfigMapper.queryListPage(params);

        // 创建并返回 Page 对象
        return new Page<SysMeConfig>(page, size, total).setRecords(records);
    }

    /**
     * 新增数据
     *
     * @param boy
     */
    public void add(SysMeConfig boy) {
        //校验数据
        this.check(boy);
        //获取机器编号 检验机器编号的真实性
        String cutrecordNo = boy.getTblMachineNo();
        Long count = cutpieceMapper.selectCount(new LambdaQueryWrapper<Cutpiece>().eq(Cutpiece::getTblMachineNo, cutrecordNo));
        Assert.isTrue(count > 0, "500-机器编号不存在");
        //通过编码去查询配置表防止出现重复机器编号
        LambdaQueryWrapper<SysMeConfig> wr = new LambdaQueryWrapper<>();
        wr.eq(SysMeConfig::getTblMachineNo, cutrecordNo);
        Long aLong = sysMeConfigMapper.selectCount(wr);
        Assert.isTrue(aLong == 0, "500-机器编号已存在");
        //新增数据
        boy.setCreateTime(new Date());
        sysMeConfigMapper.insert(boy);
    }

    /**
     * 修改数据
     *
     * @param boy
     */
    public void updateBoy(SysMeConfig boy) {
        //校验数据
        this.check(boy);
        //获取机器编码
        String cutrecordNo = boy.getTblMachineNo();
        //校验机器编码的真实性
        Long count = cutpieceMapper.selectCount(new LambdaQueryWrapper<Cutpiece>().eq(Cutpiece::getTblMachineNo, cutrecordNo));
        Assert.isTrue(count > 0, "500-机器编号不存在");
        boy.setUpdateTime(new Date());
        sysMeConfigMapper.updateById(boy);
    }

    /**
     * 根据id删除数据
     *
     * @param ids
     */
    public void delete(Long[] ids) {
        //遍历ids 判断数据是否存在
        for (Long id : ids) {
            SysMeConfig sysMeConfig = sysMeConfigMapper.selectById(id);
            Assert.isTrue(sysMeConfig != null, "500-数据不存在");
            //删除数据
            sysMeConfigMapper.deleteById(id);
        }

    }


    /**
     * 新增数据时校验数据
     *
     * @param boy
     */
    private void check(SysMeConfig boy) {
        Assert.isTrue(StringUtils.isNotBlank(boy.getTblMachineNo()), "500-机器编号不能为空");
        Assert.isTrue(boy.getCapacity() != null && boy.getCapacity() > 0, "500-产能不能为空");
        Assert.isTrue(boy.getConfiTime() != null, "500-上班时间不能为空");
        Assert.isTrue(StringUtils.isNotBlank(boy.getMachineName()), "500-机器名称不能为空");
    }


}
